package com.ruoyi.ss.device.service.impl;

import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.core.redis.RedisLock;
import com.ruoyi.common.core.redis.enums.RedisLockKey;
import com.ruoyi.common.enums.LoginType;
import com.ruoyi.common.enums.UserType;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.*;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.iot.domain.IotDeviceInfo;
import com.ruoyi.iot.domain.response.CommandResponse;
import com.ruoyi.iot.enums.IotHttpStatus;
import com.ruoyi.iot.service.IotService;
import com.ruoyi.ss.dashboard.vo.BillCountVo;
import com.ruoyi.ss.device.domain.Device;
import com.ruoyi.ss.device.domain.DeviceBO;
import com.ruoyi.ss.device.domain.DeviceCountVO;
import com.ruoyi.ss.device.domain.DeviceQuery;
import com.ruoyi.ss.device.domain.dto.DeviceBatchUpdateModelDTO;
import com.ruoyi.ss.device.domain.dto.DeviceRegisterDTO;
import com.ruoyi.ss.device.domain.dto.DeviceWifiDTO;
import com.ruoyi.ss.device.domain.enums.DeviceOnlineStatus;
import com.ruoyi.ss.device.domain.enums.DevicePowerStatus;
import com.ruoyi.ss.device.domain.enums.DeviceStatus;
import com.ruoyi.ss.device.domain.vo.DeviceMacSnVO;
import com.ruoyi.ss.device.domain.vo.DeviceVO;
import com.ruoyi.ss.device.mapper.DeviceMapper;
import com.ruoyi.ss.device.service.DeviceAssembler;
import com.ruoyi.ss.device.service.DeviceService;
import com.ruoyi.ss.device.service.DeviceValidator;
import com.ruoyi.ss.deviceBindRecord.service.ISmDeviceBindRecordService;
import com.ruoyi.ss.deviceSuit.service.DeviceSuitConverter;
import com.ruoyi.ss.deviceSuit.service.DeviceSuitService;
import com.ruoyi.ss.model.domain.SmModelVO;
import com.ruoyi.ss.model.domain.enums.ModelTag;
import com.ruoyi.ss.model.service.ModelService;
import com.ruoyi.ss.record.time.domain.enums.RecordTimeType;
import com.ruoyi.ss.record.time.service.IRecordTimeService;
import com.ruoyi.ss.record.time.service.RecordTimeConverter;
import com.ruoyi.ss.store.domain.StoreVo;
import com.ruoyi.ss.store.service.StoreService;
import com.ruoyi.ss.suit.domain.enums.SuitFeeType;
import com.ruoyi.ss.timeBill.domain.TimeBillQuery;
import com.ruoyi.ss.timeBill.domain.TimeBillVO;
import com.ruoyi.ss.timeBill.service.TimeBillService;
import com.ruoyi.ss.transactionBill.domain.TransactionBillQuery;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillStatus;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillType;
import com.ruoyi.ss.transactionBill.domain.vo.TransactionBillVO;
import com.ruoyi.ss.transactionBill.service.TransactionBillService;
import com.ruoyi.ss.user.domain.SmUserVo;
import com.ruoyi.ss.user.service.ISmUserService;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.support.TransactionTemplate;

import java.math.BigDecimal;
import java.time.Duration;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 设备Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-02-21
 */
@Service
@Slf4j
public class DeviceServiceImpl implements DeviceService
{
    @Autowired
    private DeviceMapper deviceMapper;

    @Autowired
    private IotService iotService;

    @Autowired
    private ISmDeviceBindRecordService smDeviceBindRecordService;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    @Autowired
    private DeviceAssembler deviceAssembler;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private StoreService storeService;

    @Autowired
    private IRecordTimeService recordTimeService;

    @Autowired
    private RecordTimeConverter recordTimeConverter;

    @Autowired
    private ISmUserService userService;

    @Autowired
    private DeviceValidator deviceValidator;

    @Autowired
    private DeviceSuitConverter deviceSuitConverter;

    @Autowired
    private TransactionBillService transactionBillService;

    @Autowired
    private RedisLock redisLock;

    @Autowired
    private ModelService modelService;

    @Autowired
    private TimeBillService timeBillService;

    @Autowired
    private DeviceSuitService deviceSuitService;

    private final String WIFI_MAC = "D8";
    private final String FOUR_G_MAC = "4C";

    /**
     * 查询设备
     *
     * @param deviceId 设备主键
     * @return 设备
     */
    @Override
    public DeviceVO selectSmDeviceByDeviceId(Long deviceId)
    {
        return deviceMapper.selectSmDeviceByDeviceId(deviceId);
    }

    /**
     * 查询设备列表
     *
     * @param smDevice 设备
     * @return 设备
     */
    @Override
    public List<DeviceVO> selectSmDeviceList(DeviceQuery smDevice) {
        return deviceMapper.selectSmDeviceList(smDevice);
    }

    /**
     * 新增设备
     * @param data 设备
     * @return 结果
     */
    @Override
    public int insertSmDevice(Device data) {
        ServiceUtil.assertion(data == null, "参数错误，不允许为空");
        ServiceUtil.assertion(StringUtils.isBlank(data.getDeviceNo()), "参数错误，SN不允许为空");
        ServiceUtil.assertion(StringUtils.isBlank(data.getMac()), "参数错误，MAC不允许为空");

        String key = data.getDeviceNo();
        ServiceUtil.assertion(!redisLock.lock(RedisLockKey.ADD_DEVICE, key), "当前新增设备过于频繁，请稍后再试");
        try {

            SmModelVO model = modelService.selectSmModelByModelId(data.getModelId());
            ServiceUtil.assertion(model == null, "型号不存在");
            ServiceUtil.assertion(StringUtils.isBlank(model.getProductId()), "型号产品ID未配置，请联系管理员处理");

            data.setCreateTime(DateUtils.getNowDate());
            data.setStatus(DeviceStatus.NORMAL.getStatus());
            data.setOnlineStatus(DeviceOnlineStatus.OFFLINE.getStatus());

            Integer result = transactionTemplate.execute(status -> {
                // 创建设备
                int insert = deviceMapper.insertSmDevice(data);
                ServiceUtil.assertion(insert != 1, "新增设备失败");

                // 后校验
                DeviceVO vo = this.selectSmDeviceByDeviceId(data.getDeviceId());
                deviceValidator.afterCheck(vo);

                // 创建OneNet设备1
                int code = iotService.create(data.getMac(), model.getProductId());
                ServiceUtil.assertion(!IotHttpStatus.SUCCESS.equalCode(code) && !IotHttpStatus.DEVICE_EXIST.equalCode(code), "MAC-1注册失败");
                // 创建OneNet设备2
                if (StringUtils.hasText(data.getMac2())) {
                    code = iotService.create(data.getMac2(), model.getProductId());
                    ServiceUtil.assertion(!IotHttpStatus.SUCCESS.equalCode(code) && !IotHttpStatus.DEVICE_EXIST.equalCode(code), "MAC-2注册失败");
                }
                return insert;
            });
            return result == null ? 0 : result;
        } finally {
            redisLock.unlock(RedisLockKey.ADD_DEVICE, key);
        }
    }

    /**
     * 修改设备
     *
     * @param data 设备
     * @return 结果
     */
    @Override
    @Transactional
    public int updateSmDevice(DeviceBO data) {

        data.setUpdateTime(DateUtils.getNowDate());

        // 修改数据库
        Integer result = transactionTemplate.execute(status -> {
            int update = deviceMapper.updateSmDevice(data);
            ServiceUtil.assertion(update != 1, "更新设备失败");

            // 后校验
            DeviceVO vo = this.selectSmDeviceByDeviceId(data.getDeviceId());
            deviceValidator.afterCheck(vo);

            // 更新套餐关联
            if (data.getSuitIds() != null) {
                // 删除设备套餐关联
                deviceSuitService.deleteByDeviceId(data.getDeviceId());
                // 新增关联
                deviceSuitService.batchInsert(deviceSuitConverter.toPo(data));
            }

            return update;
        });

        return result == null ? 0 : result;
    }

    /**
     * 批量删除设备
     *
     * @param deviceIds 需要删除的设备主键
     * @return 结果
     */
    @Override
    public int deleteSmDeviceByDeviceIds(Long[] deviceIds)
    {
        this.validatePreDelete(Arrays.asList(deviceIds));
        return deviceMapper.deleteSmDeviceByDeviceIds(deviceIds);
    }

    /**
     * 真正的删除校验
     */
    private void validatePreDelete(List<Long> deviceIds) {
        ServiceUtil.assertion(CollectionUtils.isEmpty(deviceIds), "设备不能为空");
        DeviceQuery dto = new DeviceQuery();
        dto.setDeviceIds(deviceIds);
        dto.setDeleted(true);
        Map<Long, DeviceVO> deviceMap = selectMap(dto);

        for (Long deviceId : deviceIds) {
            DeviceVO device = deviceMap.get(deviceId);
            ServiceUtil.assertion(device == null, String.format("设备“%s”不存在，请刷新后重试", deviceId));
        }
    }

    /**
     * 逻辑删除校验
     */
    private void validatePreLogicDelete(List<Long> deviceIds) {
        ServiceUtil.assertion(CollectionUtils.isEmpty(deviceIds), "设备不能为空");

        DeviceQuery dto = new DeviceQuery();
        dto.setDeviceIds(deviceIds);
        dto.setDeleted(false);
        // 前台用户，判断是不是他的设备
        if (LoginType.FRONT.equals(SecurityUtils.getLoginType())) {
            dto.setUserId(SecurityUtils.getUserId());
        }
        Map<Long, DeviceVO> deviceMap = selectMap(dto);

        for (Long deviceId : deviceIds) {
            DeviceVO device = deviceMap.get(deviceId);
            ServiceUtil.assertion(device == null, String.format("设备“%s”不存在，请刷新后重试", deviceId));
        }

        // 拉取最新数据
        this.pullDeviceInfo(deviceIds);

        // 查询是否有未完成的订单
        TransactionBillQuery query = new TransactionBillQuery();
        query.setDeviceIds(deviceIds);
        query.setIsFinished(false);
        query.setType(TransactionBillType.RECHARGE.getType());
        int billCount = transactionBillService.selectSimpleCount(query);
        ServiceUtil.assertion(billCount > 0, "设备存在未完成订单，无法删除");
    }

    /**
     * 条件查询列表，并转为Map
     */
    private Map<Long, DeviceVO> selectMap(DeviceQuery dto) {
        List<DeviceVO> list = deviceMapper.selectSmDeviceList(dto);
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyMap();
        }
        return list.stream().collect(Collectors.toMap(DeviceVO::getDeviceId, item -> item));
    }

    @Override
    public int selectCount(DeviceQuery dto) {
        return deviceMapper.selectCount(dto);
    }

    @Override
    public List<DeviceVO> selectSmDeviceByStoreId(Long storeId) {
        DeviceQuery dto = new DeviceQuery();
        dto.setStoreId(storeId);
        dto.setDeleted(false);
        return deviceMapper.selectSmDeviceList(dto);
    }


    @Override
    public boolean addTimeByUser(Long deviceId, long seconds, boolean withIot, String reason) {
        boolean addTime = this.addTime(deviceId, seconds, withIot);

        if (addTime) {
            // 记录下操作日志
            LoginUser loginUser = SecurityUtils.getLoginUser();
            scheduledExecutorService.schedule(() -> {
                DeviceVO device = deviceMapper.selectSmDeviceByDeviceId(deviceId);
                recordTimeService.insertRecordTime(recordTimeConverter.toRecordTime(device, seconds, reason, loginUser, RecordTimeType.TIME.getType()));
            }, 0, TimeUnit.SECONDS);
        }

        return addTime;
    }

    @Override
    public int register(DeviceRegisterDTO dto) {
        if (dto == null) {
            return 0;
        }
        SmModelVO model = modelService.selectSmModelByModelId(dto.getModelId());
        ServiceUtil.assertion(model == null, "型号不存在");

        // 添加
        Device device = new Device();
        device.setDeviceName("未命名");
        device.setMac(dto.getMac());
        device.setDeviceNo(dto.getSn());
        device.setModelId(dto.getModelId());
        device.setServiceType(model.getServiceType());
        device.setServiceRate(model.getServiceRate());

        // 若为双模版本的型号，则注册4G或WIFI版本MAC
        if (ModelTag.isTwoMac(model.getTags())) {
            if (device.getMac().endsWith(FOUR_G_MAC)) {
                device.setMac2(StringUtils.replaceLast(device.getMac(), FOUR_G_MAC, WIFI_MAC));
            } else {
                // 替换最后两位为4G MAC
                String substring = device.getMac().substring(0, device.getMac().length() - 2);
                device.setMac2(substring + FOUR_G_MAC);
            }
        }

        return this.insertSmDevice(device);
    }

    @Override
    public List<DeviceMacSnVO> selectMacSnList(DeviceQuery query) {
        return deviceMapper.selectMacSnList(query);
    }

    @Override
    public DeviceVO selectOne(DeviceQuery query) {
        return deviceMapper.selectOne(query);
    }

    @Override
    public List<DeviceVO> selectUsingDevice(Long userId) {
        // 查询正在使用中的订单
        List<TransactionBillVO> usingBills = transactionBillService.selectUsingBill(userId);
        List<Long> deviceIds = usingBills.stream().map(TransactionBillVO::getDeviceId).distinct().collect(Collectors.toList());
        // 查询设备列表
        return this.selectListByIds(deviceIds);
    }

    @Override
    public List<Long> selectDeviceUsingUser(Long deviceId) {
        // 正在使用的用户ID列表
        Set<Long> userIds = new HashSet<>();

        // 充值订单
        TransactionBillQuery query = new TransactionBillQuery();
        query.setDeviceId(deviceId);
        List<TransactionBillVO> transactionList = transactionBillService.selectUsingBill(query);
        if (CollectionUtils.isNotEmptyElement(transactionList)) {
            userIds.addAll(transactionList.stream().map(TransactionBillVO::getUserId).collect(Collectors.toList()));
        }

        // 时长订单
        TimeBillQuery timeQuery = new TimeBillQuery();
        timeQuery.setDeviceId(deviceId);
        List<TimeBillVO> timeBillList = timeBillService.selectUsingList(timeQuery);
        if (CollectionUtils.isNotEmptyElement(timeBillList)) {
            userIds.addAll(timeBillList.stream().map(TimeBillVO::getUserId).collect(Collectors.toList()));
        }

        return new ArrayList<>(userIds);
    }

    /**
     * 开关设备
     */
    @Override
    public boolean switchDevice(Long deviceId, boolean open) {
        DeviceVO device = selectSmDeviceByDeviceId(deviceId);
        ServiceUtil.assertion(device == null, "设备不存在");
        if (open) {
            return iotService.open(device);
        } else {
            return iotService.close(device);
        }
    }

    @Override
    public int batchUpdateModel(DeviceBatchUpdateModelDTO dto) {
        return deviceMapper.batchUpdateModel(dto);
    }

    @Override
    public int updateServiceRate(DeviceVO data) {
        return deviceMapper.updateServiceRate(data);
    }

    @Override
    public int renewalRentTime(Long deviceId, int time, TimeUnit timeUnit) {
        long seconds = timeUnit.toSeconds(time);
        return deviceMapper.renewalRentTime(deviceId, seconds);
    }

    @Override
    public void pullDeviceInfo(Long deviceId) {
        pullDeviceInfo(Collections.singletonList(deviceId));
    }

    @Override
    public int clearTimeAndEle(DeviceVO device, boolean required) {
        if (device == null) {
            return 0;
        }

        // 归零时长
        int resetTime = this.resetTime(device, required);

        // 归零剩余电量
        int resetEle = this.resetEle(device, required);

        return 1;
    }

    @Override
    public int resetTime(DeviceVO device, boolean required) {
        LocalDateTime now = LocalDateTime.now();
        Integer result = transactionTemplate.execute(status -> {
            // 归零时间，将过期时间设置为现在
            Device form = new Device();
            form.setDeviceId(device.getDeviceId());
            form.setExpireTime(now);
            form.setStatus(DeviceStatus.NORMAL.getStatus());
            int update = deviceMapper.updateSmDevice(form);
            ServiceUtil.assertion(update != 1, "修改设备时间失败");

            // 物联网设备归零
            try {
                CommandResponse res = iotService.setTime(device, 0L);
                ServiceUtil.assertion(res == null, "设备归零失败，返回值为空");
                ServiceUtil.assertion(!res.isSuccess(), "设备归零失败，请检查设备是否在线或联系管理员");

//                boolean close = iotService.close(device);
//                ServiceUtil.assertion( !close, "设备关闭失败，请检查设备是否在线或联系管理员");
            } catch (Exception e) {
                log.warn("设备归零失败:{}, {}, {}", device.getDeviceId(), device.getDeviceNo(), e.getMessage());
                if (required) {
                    throw e;
                }
            }

            return update;
        });

        // 归零记录
        LoginUser loginUser = SecurityUtils.getLoginUser();
        if (result != null && result == 1 ) {
            scheduledExecutorService.schedule(() -> {
                // 设备剩余时长
                Duration duration = Duration.between(now, device.getExpireTime());
                long seconds = duration.getSeconds() > 0 ? duration.getSeconds() : 0;
                recordTimeService.insertRecordTime(recordTimeConverter.toRecordTime(device, -seconds, "设备归零", loginUser, RecordTimeType.TIME.getType()));
            }, 0, TimeUnit.SECONDS);
        }

        return result == null ? 0 : result;
    }

    @Override
    public int resetEle(DeviceVO device, boolean required) {
        if (device == null) {
            return 0;
        }

        Integer result = transactionTemplate.execute(status -> {
            // 更新剩余电量
            Device data = new Device();
            data.setDeviceId(device.getDeviceId());
            data.setSurplusEle(BigDecimal.ZERO);
            int update = deviceMapper.updateSmDevice(data);
            ServiceUtil.assertion(update != 1, "更新剩余电量失败");

            // 直接设置电量为0
            try {
                CommandResponse res = iotService.setEle(device, BigDecimal.valueOf(0));
                ServiceUtil.assertion(res == null, "归零电量失败，返回值为空");
                ServiceUtil.assertion(!res.isSuccess(), "归零电量失败，请检查设备是否在线或联系管理员");
            } catch (Exception e) {
                if (required) {
                    throw e;
                }
            }

            return update;
        });

        // 归零记录
        LoginUser loginUser = SecurityUtils.getLoginUser();
        if (result != null && result == 1 ) {
            scheduledExecutorService.schedule(() -> {
                recordTimeService.insertRecordTime(recordTimeConverter.toRecordTime(device, device.getSurplusEle().negate().intValue(), "设备归零", loginUser, RecordTimeType.ELE.getType()));
            }, 0, TimeUnit.SECONDS);
        }

        return result == null ? 0 : result;
    }

    @Override
    public int resetEle(Long deviceId, boolean required) {
        // 获取设备信息
        DeviceVO device = selectSmDeviceByDeviceId(deviceId);
        return resetEle(device, required);
    }

    @Override
    public int resetEleWithBill(Long deviceId, boolean requiredIot) {
        if (deviceId == null) {
            return 0;
        }
        // 拉取设备信息
        pullDeviceInfo(deviceId);

        DeviceVO device = selectSmDeviceByDeviceId(deviceId);
        ServiceUtil.assertion(device == null, "设备不存在");

        // 尝试关闭该设备未结束的所有订单
        TransactionBillQuery query = new TransactionBillQuery();
        query.setStatusList(TransactionBillStatus.canClose());
        query.setDeviceId(device.getDeviceId());
        query.setIsFinished(false);   // 未结束的订单都会被关闭
        query.setSuitFeeTypes(SuitFeeType.rechargeCountList());
        List<TransactionBillVO> billList = transactionBillService.selectSmTransactionBillList(query);
        try {
            int closeCount = transactionBillService.batchCloseBillByDevice(billList, false, device);
            ServiceUtil.assertion(closeCount != billList.size(), "关闭订单失败:closeCount =" + closeCount);
        } catch (Exception e) {
            log.error("关闭订单失败:{}" , e.getMessage());
        }

        // 发送命令
        int reset = this.resetEle(device, requiredIot);
        ServiceUtil.assertion(reset != 1, "设备电量归零失败");

        return 1;
    }

    @Override
    public boolean addEle(Long deviceId, BigDecimal ele, boolean withIot, String reason) {
        DeviceVO device = selectSmDeviceByDeviceId(deviceId);
        if (device == null) {
            return false;
        }

        if (withIot) {
            CommandResponse res = iotService.addEle(device, ele);
            return res.isSuccess();
        }
        return true;
    }

    @Override
    public int updateLastRecoverTime(Long deviceId, Long lastRecoverTime) {
        Device data = new Device();
        data.setDeviceId(deviceId);
        data.setLastRecoverTime(lastRecoverTime);
        return deviceMapper.updateSmDevice(data);
    }

    @Override
    public int setTime(DeviceVO device, LocalDateTime expireTime, boolean withIot, int tryCount) {
        if (device == null || StringUtils.isBlank(device.getMac()) || expireTime == null) {
            return 0;
        }

        Integer result = transactionTemplate.execute(status -> {
            Device data = new Device();
            data.setDeviceId(device.getDeviceId());
            data.setExpireTime(expireTime);
            int update = deviceMapper.updateSmDevice(data);
            ServiceUtil.assertion(update != 1, "更新数据失败");

            if (withIot) {
                Duration between = Duration.between(LocalDateTime.now(), expireTime);
                long seconds = between.getSeconds();
                if (seconds > 0) {
                    CommandResponse res = iotService.trySetTime(device, seconds, tryCount);
                    if (!res.isSuccess()) {
                        log.error("操作设备失败：{}", res.getMsg());
                    }
                    return res.isSuccess() ? 1 : 0;
                }
            }

            return update;
        });

        return result == null ? 0 : 1;
    }

    @Override
    public int setWifi(DeviceWifiDTO dto) {
        DeviceVO device = selectSmDeviceByDeviceId(dto.getDeviceId());
        if (device == null) {
            return 0;
        }
        CommandResponse res = iotService.setWifi(device, dto.getWifiName(), dto.getWifiPwd());
        ServiceUtil.assertion(!res.isSuccess(), "设置WIFI失败：" + res.getMsg());

        return 1;
    }


    @Override
    public boolean addTime(Long deviceId, long seconds, boolean withIot) {
        ServiceUtil.assertion( seconds < 0, "增加的时长不允许小于0");

        DeviceVO device = deviceMapper.selectSmDeviceByDeviceId(deviceId);
        ServiceUtil.assertion(device == null, "设备不存在");
        ServiceUtil.assertion(!StringUtils.hasText(device.getMac()), "设备MAC号为空");
        ServiceUtil.assertion(DeviceStatus.FIXING.getStatus().equals(device.getStatus()), "设备正在维修中，无法使用");

        Boolean result = transactionTemplate.execute(status -> {
            // 更新数据库时长
            int updateCount = deviceMapper.addTime(deviceId, seconds);
            ServiceUtil.assertion(updateCount != 1, "增加时长失败，请刷新后重试");

            // 修改状态为使用中
            changeStatus(deviceId, DeviceStatus.USING);

            // 物联网设备增加时长
            if (withIot) {
                DeviceVO newDevice = selectSmDeviceByDeviceId(deviceId);
                long betweenSeconds = Duration.between(LocalDateTime.now(), newDevice.getExpireTime()).getSeconds();
                if (betweenSeconds > 0) {
                    CommandResponse rechargeResult = iotService.setTime(device, betweenSeconds);
                    ServiceUtil.assertion(!rechargeResult.isSuccess(), "设备充值失败，请检查设备是否在线");
                }
            }
            return Boolean.TRUE;
        });

        // 是否成功
        boolean success = result != null && result;
        if (success) {
            // 拉取设备信息
            this.pullDeviceInfoAsync(Collections.singletonList(deviceId), 3, TimeUnit.SECONDS);

            // 时长结束后修改设备状态
//            scheduledExecutorService.schedule(()-> {
//                freshStatus(deviceId);
//            }, seconds, TimeUnit.SECONDS);
        }

        return success;
    }


//    private void freshStatus(Long deviceId) {
//        DeviceVO device = deviceMapper.selectSmDeviceByDeviceId(deviceId);
//        freshStatus(device);
//    }

    @Override
    public void freshStatus(DeviceVO device) {
        if (device == null || device.getExpireTime() == null) {
            return;
        }
        if (!DeviceStatus.USING.getStatus().equals(device.getStatus())) {
            return;
        }
        LocalDateTime now = LocalDateTime.now();
        Duration between = Duration.between(now, device.getExpireTime());

        // 如果已过期，则调整设备为正常状态
        int i = deviceMapper.changeTimeoutStatus(device.getDeviceId());
        if (i == 1) {
            return;
        }

//        // 如果没有过期，则延迟至过期时间继续查询
//        if (between.getSeconds() > 0) {
//            scheduledExecutorService.schedule(() -> {
//                freshStatus(device.getDeviceId());
//            }, between.getSeconds() , TimeUnit.SECONDS);
//        }
    }

    /**
     * 设备心跳，用于检测是否在线
     */
    @Override
    public void deviceHeartBeat() {
        log.info("device heart beat running...");
        List<DeviceVO> list = deviceMapper.selectSmDeviceList(new DeviceQuery());
        if (CollectionUtils.isEmpty(list)) {
            log.info("device list is empty");
            return;
        }
        LocalDateTime now = LocalDateTime.now();

        for (DeviceVO device : list) {
            if (StringUtils.hasText(device.getMac())) {
                String status = iotService.getOnlineStatus(device).getStatus();
//                log.info("device: {} {} online status is {}", device.getDeviceId(), device.getMac(), status);
                device.setOnlineStatus(status);
                if (DeviceOnlineStatus.ONLINE.getStatus().equals(status)) {
                    device.setLastOnlineTime(now);
                }
            } else {
                device.setOnlineStatus(DeviceOnlineStatus.OFFLINE.getStatus());
                log.info("device {} mac is empty", device.getDeviceId());
            }
        }
        int i = deviceMapper.batchUpdateOnlineStatus(list);
        log.info("update count: {}", i);
    }


    private int changeStatus(Long deviceId, DeviceStatus status) {
        Device device = new Device();
        device.setDeviceId(deviceId);
        device.setStatus(status.getStatus());
        return deviceMapper.updateSmDevice(device);
    }

    @Override
    public boolean logicDel(List<Long> deviceIds) {
        this.validatePreLogicDelete(deviceIds);
        return deviceMapper.logicDel(deviceIds) > 0;
    }

    @Override
    public Map<Long, List<DeviceVO>> selectGroupByModelIds(List<Long> modelIds) {
        if (CollectionUtils.isEmpty(modelIds)) {
            return Collections.emptyMap();
        }
        DeviceQuery dto = new DeviceQuery();
        dto.setModelIds(modelIds);
        List<DeviceVO> list = deviceMapper.selectSmDeviceList(dto);
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyMap();
        }
        return list.stream().collect(Collectors.groupingBy(DeviceVO::getModelId));
    }

    /**
     * 切换设备通电状态（通/断电）
     * @param deviceId 设备id
     * @param status 通电状态
     */
    @Override
    public boolean changePowerStatus(Long deviceId, DevicePowerStatus status) {
        DeviceVO device = deviceMapper.selectSmDeviceByDeviceId(deviceId);
        ServiceUtil.assertion(device == null || device.getDeleted(), "设备不存在");
        ServiceUtil.assertion(!StringUtils.hasText(device.getMac()), "设备Mac号为空");
        ServiceUtil.assertion(!UserUtil.hasFrontUser(device.getUserId()), "该设备不是您的，无法进行该操作" );

        if (DevicePowerStatus.ON.equals(status)) {
            iotService.open(device);
        } else if (DevicePowerStatus.OFF.equals(status)) {
            iotService.close(device);
        } else {
            throw new ServiceException("不支持的操作");
        }

        // 修改数据库通断电状态
        Device changePowerForm = new Device();
        changePowerForm.setDeviceId(deviceId);
        changePowerForm.setPowerStatus(status.getStatus());
        deviceMapper.updateSmDevice(changePowerForm);

        return true;
    }

    /**
     * 拉取设备信息，并保存到数据库
     * @param deviceIds 设备id列表（为空则查询所有）
     */
    @Override
    @SneakyThrows
    public void pullDeviceInfo(List<Long> deviceIds) {
        // 查询设备列表
        DeviceQuery dto = new DeviceQuery();
        dto.setDeviceIds(deviceIds);
        List<DeviceVO> list = deviceMapper.selectSmDeviceList(dto);
        this.pullDeviceInfoList(list);
    }

    @Override
    public void pullDeviceInfoList(List<DeviceVO> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }
        LocalDateTime now = LocalDateTime.now();

        for (DeviceVO device : list) {
            try {
                if (StringUtils.isAnyBlank(device.getMac(), device.getModelProductId())) {
                    continue;
                }
                // 获取数据点信息
                IotDeviceInfo deviceInfo = iotService.getDeviceInfo(device);

                // 更新设备信息
                device.setDeviceId(device.getDeviceId());
                if (deviceInfo != null) {
                    device.setLastPullTime(deviceInfo.getAt());
                    device.setTotalElectriQuantity(deviceInfo.getW());
                    device.setPowerStatus(deviceInfo.getS());
                    device.setRemainTime(deviceInfo.getTime());
                    device.setRealTimePower(deviceInfo.getP());
                    device.setVoltage(deviceInfo.getV());
                    device.setElectricity(deviceInfo.getA());

                    // 是否有WIFI
                    if (ModelTag.hasTag(device.getModelTags(), ModelTag.WIFI)) {
                        device.setWifi(deviceInfo.getWifi());
                    }

                    // 是否有电量
                    if (ModelTag.hasTag(device.getModelTags(), ModelTag.ELE)) {
                        device.setSurplusEle(deviceInfo.getM());
                    } else {
                        device.setSurplusEle(BigDecimal.ZERO);
                    }
                }

                // 判断设备是否正在使用
                // 设备过期时间 > 当前时间，则正在使用
                boolean hasTime =  device.getExpireTime() != null && device.getExpireTime().isAfter(now);
                // 若当前设备有电量，则正在使用
                boolean hasEle = device.getSurplusEle() != null && device.getSurplusEle().compareTo(BigDecimal.ZERO) > 0;
                // 若开关开启，则正在使用
                boolean hasOpen = DevicePowerStatus.ON.getStatus().equals(device.getPowerStatus());
                if (hasTime || hasEle || hasOpen) {
                    device.setStatus(DeviceStatus.USING.getStatus());
                } else {
                    device.setStatus(DeviceStatus.NORMAL.getStatus());
                }

                // 是否在线
                String onlineStatus = iotService.getOnlineStatus(device).getStatus();
                device.setOnlineStatus(onlineStatus);
                if (DeviceOnlineStatus.ONLINE.getStatus().equals(onlineStatus)) {
                    device.setLastOnlineTime(now);
                }

                this.updateIotInfo(device);
            } catch (Exception e) {
                log.error("更新设备信息失败，deviceId:{}", device.getDeviceId(), e);
            }
        }
    }

    /**
     * 更新物联网信息
     */
    private int updateIotInfo(Device device) {
        Device data = new Device();
        data.setDeviceId(device.getDeviceId());
        data.setLastPullTime(device.getLastPullTime());
        data.setTotalElectriQuantity(device.getTotalElectriQuantity());
        data.setPowerStatus(device.getPowerStatus());
        data.setRemainTime(device.getRemainTime());
        data.setRealTimePower(device.getRealTimePower());
        data.setVoltage(device.getVoltage());
        data.setElectricity(device.getElectricity());
        data.setWifi(device.getWifi());
        data.setSurplusEle(device.getSurplusEle());
        data.setStatus(device.getStatus());
        data.setOnlineStatus(device.getOnlineStatus());
        data.setLastOnlineTime(device.getLastOnlineTime());
        return deviceMapper.updateSmDevice(device);
    }

    /**
     * 异步延迟执行拉取设备信息
     * @param deviceIds 设备id列表
     * @param delay 延迟时间
     * @param unit 时间单位
     */
    private void pullDeviceInfoAsync(List<Long> deviceIds, long delay, TimeUnit unit) {
        scheduledExecutorService.schedule(() -> {
            log.info("异步拉取设备信息 start");
            this.pullDeviceInfo(deviceIds);
            log.info("异步拉取设备信息 end");
        }, delay, unit);
    }



    @Override
    public String bindBySn(Long storeId, Long userId, String deviceNo) {
        DeviceVO device = selectByDeviceNo(deviceNo);
        return this.bind(storeId, userId, device);
    }

    @Override
    public String bindByMac(Long storeId, Long userId, String mac) {
        DeviceVO device = selectByMac(mac);
        return this.bind(storeId, userId, device);
    }

    private String bind(Long storeId, Long userId, DeviceVO device) {
        ServiceUtil.assertion(device == null, "设备未录入");
        // 如果绑定的用户是原来的用户，则直接返回MAC
        if (device.getUserId() != null) {
            if (Objects.equals(device.getUserId(), userId)) {
                return device.getMac();
            } else {
                throw new ServiceException("该设备已被绑定");
            }
        }
        // 如果绑定的是原来的店铺，则直接返回MAC
        if (device.getStoreId() != null) {
            if (Objects.equals(device.getStoreId(), storeId)) {
                return device.getMac();
            } else {
                throw new ServiceException("该设备已被绑定");
            }
        }

        ServiceUtil.assertion(device.getLimitRechargeTime() != null
                && device.getLimitRechargeTime().isAfter(LocalDateTime.now()),
                "设备无法绑定：" + device.getLimitRechargeReason()
        );

        // 查询用户
        SmUserVo user = userService.selectSimpleById(userId);
        ServiceUtil.assertion(user == null, "用户不存在");

        // 查询店铺
        if (storeId != null) {
            StoreVo store = storeService.selectSmStoreById(storeId);
            ServiceUtil.assertion(store == null, "店铺不存在");
            ServiceUtil.assertion(!Objects.equals(store.getUserId(), userId), "该店铺不属于该用户");
        }

        transactionTemplate.execute(status -> {
            // 绑定
            int updateCount = deviceMapper.bind(device.getDeviceId(), storeId, userId);
            ServiceUtil.assertion(updateCount != 1, "当前设备信息已变更，请刷新后重试");

            // 记录绑定记录
            int record = smDeviceBindRecordService.record(userId, device.getDeviceId());
            ServiceUtil.assertion(record != 1, "添加绑定记录失败");

            // 用户设置为商户
            int changeType = userService.changeType(userId, UserType.MCH);
            ServiceUtil.assertion(changeType != 1, "修改用户类型失败");

            return updateCount;
        });

        return device.getMac();
    }

    /**
     * 根据mac号查询数据
     * @param mac
     * @return
     */
    @Override
    public DeviceVO selectByMac(String mac) {
        if (StringUtils.isBlank(mac)) {
            return null;
        }
        return deviceMapper.selectSimpleSmDeviceByMac(mac);
    }

    @Override
    public int resetTimeWithBill(Long deviceId, boolean requiredIot) {
        ServiceUtil.assertion(deviceId == null, "设备id不能为空");

        // 拉取最新设备信息
        pullDeviceInfo(deviceId);

        // 获取设备信息
        DeviceVO device = deviceMapper.selectSmDeviceByDeviceId(deviceId);
        ServiceUtil.assertion(device == null || device.getDeleted(), "设备不存在");
        ServiceUtil.assertion(!UserUtil.hasFrontUser(device.getUserId()), "您不是该设备的商户，无法进行该操作");

        // 尝试关闭该设备未结束的所有订单
        TransactionBillQuery query = new TransactionBillQuery();
        query.setStatusList(TransactionBillStatus.canClose());
        query.setDeviceId(device.getDeviceId());
        query.setIsFinished(false);   // 未结束的订单都会被关闭
        query.setSuitFeeTypes(SuitFeeType.rechargeTimeList());
        List<TransactionBillVO> billList = transactionBillService.selectSmTransactionBillList(query);
        try {
            int closeCount = transactionBillService.batchCloseBillByDevice(billList, false, device);
            ServiceUtil.assertion(closeCount != billList.size(), "关闭订单失败:closeCount =" + closeCount);
        } catch (Exception e) {
            log.error("关闭订单失败: {}", e.getMessage());
        }

        // 发送命令
        int reset = this.resetTime(device, requiredIot);
        ServiceUtil.assertion(reset != 1, "归零失败");

        return reset;
    }

    /**
     * 设备是否已经被绑定
     *
     * @param mac
     * @return
     */
    @Override
    public int bindCodeByMac(String mac) {
        if (StringUtils.isBlank(mac)) {
            return Constants.DEVICE_NOT_FOUND;
        }
        DeviceVO device = deviceMapper.selectSimpleSmDeviceByMac(mac);
        return isBindCode(device);
    }

    private int isBindCode(DeviceVO device) {
        if (device == null) {
            return Constants.DEVICE_NOT_FOUND;
        }
        if (device.getUserId() == null) {
            return Constants.DEVICE_NOT_BIND;
        } else {
            return Constants.DEVICE_BINDED;
        }
    }

    /**
     * 根据SN码查询设备是否已经绑定
     *
     * @param deviceNo
     * @return
     */
    @Override
    public int bindCodeBySn(String deviceNo) {
        if (StringUtils.isBlank(deviceNo)) {
            return Constants.DEVICE_NOT_FOUND;
        }
        DeviceVO device = deviceMapper.selectSimpleSmDeviceBySn(deviceNo);
        return isBindCode(device);
    }

    /**
     * 更新设备服务费
     *
     * @param userId
     * @param serviceRate
     */
    @Override
    public int updateServiceRateByUserId(Long userId, BigDecimal serviceRate) {
        if (userId == null) {
            return 0;
        }
        List<StoreVo> list = storeService.selectByUserId(userId);
        if (CollectionUtils.isEmpty(list)) {
            return 0;
        }
        return deviceMapper.updateServiceRateByStores(list.stream().map(StoreVo::getStoreId).collect(Collectors.toList()), serviceRate);
    }

    @Override
    public int updateByIot(Device device) {
        device.setLastPullTime(DateUtils.getNowDate());
        return deviceMapper.updateSmDeviceByIot(device);
    }

    /**
     * ids查询列表
     *
     * @param ids
     * @param userId
     */
    @Override
    public List<DeviceVO> selectListByIds(List<Long> ids, Long userId) {
        if (CollectionUtils.isEmptyElement(ids)) {
            return Collections.emptyList();
        }
        DeviceQuery dto = new DeviceQuery();
        dto.setDeviceIds(ids);
        dto.setUserId(userId);
        return selectSmDeviceList(dto);
    }

    /**
     * 查询各店铺的设备数量,并店铺
     *
     * @param storeIds 店铺id
     */
    @Override
    public Map<Long, Integer> selectCountMapByStoreIds(List<Long> storeIds) {
        if (CollectionUtils.isEmptyElement(storeIds)) {
            return Collections.emptyMap();
        }
        List<DeviceCountVO> list = deviceMapper.selectCountByStoreIds(storeIds);
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyMap();
        }
        return list.stream().collect(Collectors.toMap(DeviceCountVO::getStoreId, DeviceCountVO::getCount));
    }

    /**
     * 条件查询数量，并店铺
     *
     * @param query 查询条件
     * @param keyMapper 键值的映射
     */
    @Override
    public <K> Map<K, Integer> selectCommonCountMap(DeviceQuery query, Function<? super DeviceCountVO, ? extends K> keyMapper ) {
        if (query == null) {
            return Collections.emptyMap();
        }
        List<DeviceCountVO> list = deviceMapper.selectCommonCount(query);
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyMap();
        }
        return list.stream().collect(Collectors.toMap(keyMapper, DeviceCountVO::getCount));
    }

    /**
     * 获取设备，包含套餐列表
     *
     * @param deviceSn
     * @return
     */
    @Override
    public DeviceVO getDeviceWithSuitList(String deviceSn) {
        if (deviceSn == null) {
            return null;
        }
        DeviceVO device = deviceMapper.selectByDeviceNo(deviceSn);
        if (device != null) {
            deviceAssembler.assembleSuitList(Collections.singletonList(device));
        }
        return device;
    }

    /**
     * 根据设备编号查询
     *
     * @param deviceNo
     */
    @Override
    public DeviceVO selectByDeviceNo(String deviceNo) {
        return deviceMapper.selectByDeviceNo(deviceNo);
    }

    @Override
    @Transactional
    public int unbind(Long deviceId) {
        DeviceVO device = deviceMapper.selectSmDeviceByDeviceId(deviceId);
        ServiceUtil.assertion(device == null || device.getDeleted(), "设备不存在");
        ServiceUtil.assertion(DeviceStatus.USING.getStatus().equals(device.getStatus()), "设备正在使用中，无法解除绑定");

        Integer result = transactionTemplate.execute(status -> {
            int updateCount = deviceMapper.unbindStore(deviceId);
            ServiceUtil.assertion(updateCount != 1, "当前设备信息已变更，请刷新后重试");

            // 删除套餐关联关系
            deviceSuitService.deleteByDeviceId(deviceId);

            return updateCount;
        });

        return result == null ? 0 : result;
    }

    /**
     * 设备绑定SN码
     *
     * @param deviceId 设备ID
     * @param sn       SN
     */
    @Override
    public int bindSn(Long deviceId, String sn) {
        Integer result = transactionTemplate.execute(status -> {
            ServiceUtil.assertion(deviceValidator.isRepeatSn(deviceId, sn), "SN重复");

            int i = deviceMapper.bindSn(deviceId, sn);
            ServiceUtil.assertion(i != 1, "设备SN已发生变化，请刷新后重试");
            return i;
        });
        return result == null ? 0 : result;
    }
}
