package com.ruoyi.ss.transactionBill.service.impl;

import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.ss.account.domain.AccountQuery;
import com.ruoyi.ss.account.domain.AccountVO;
import com.ruoyi.ss.account.service.AccountService;
import com.ruoyi.ss.channel.service.ChannelService;
import com.ruoyi.ss.channelWithdraw.domain.ChannelWithdrawVO;
import com.ruoyi.ss.channelWithdraw.service.ChannelWithdrawService;
import com.ruoyi.ss.device.domain.vo.DeviceVO;
import com.ruoyi.ss.device.service.DeviceService;
import com.ruoyi.ss.store.domain.StoreVo;
import com.ruoyi.ss.store.service.StoreService;
import com.ruoyi.ss.suit.domain.SuitVO;
import com.ruoyi.ss.suit.service.SuitAssembler;
import com.ruoyi.ss.suit.service.SuitService;
import com.ruoyi.ss.transactionBill.domain.bo.*;
import com.ruoyi.ss.transactionBill.domain.dto.*;
import com.ruoyi.ss.transactionBill.domain.vo.TransactionBillVO;
import com.ruoyi.ss.transactionBill.service.TransactionBillConverter;
import com.ruoyi.ss.user.domain.SmUserVo;
import com.ruoyi.ss.user.service.ISmUserService;
import com.ruoyi.system.service.ISysDeptService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;

/**
 * @author wjh
 * 2024/7/28
 */
@Service
public class TransactionBillConverterImpl implements TransactionBillConverter {

    @Autowired
    private DeviceService deviceService;

    @Autowired
    private ISmUserService userService;

    @Autowired
    private SuitService suitService;

    @Autowired
    private ChannelService channelService;

    @Autowired
    private AccountService accountService;

    @Autowired
    private SuitAssembler suitAssembler;

    @Autowired
    private StoreService storeService;

    @Autowired
    private ChannelWithdrawService channelWithdrawService;

    @Autowired
    private TransactionBillServiceImpl transactionBillService;

    @Autowired
    private ISysDeptService deptService;

    /**
     * 将参数转为充值BO
     *
     * @param dto
     */
    @Override
    public RechargeBO toRechargeBO(RechargeDTO dto) {
        if (dto == null) {
            return null;
        }

        DeviceVO device = deviceService.selectByDeviceNo(dto.getDeviceNo());
        ServiceUtil.assertion(device == null, "设备不存在");

        StoreVo store = storeService.selectSmStoreById(device.getStoreId());

        SmUserVo mch = userService.selectSmUserByUserId(device.getUserId());
        ServiceUtil.assertion(mch == null, "商户不存在");

        SuitVO suit = suitService.selectSuitBySuitId(dto.getSuitId());
        ServiceUtil.assertion(suit == null, "套餐不存在");
        suitAssembler.assembleDeviceList(Collections.singletonList(suit));

        SmUserVo user = userService.selectSmUserByUserId(SecurityUtils.getUserId());
        ServiceUtil.assertion(user == null, "用户不存在");
        ServiceUtil.assertion(user.getWxOpenId() == null, "用户微信openId为空");

        RechargeBO bo = new RechargeBO();
        bo.setDevice(device);
        bo.setUser(user);
        bo.setSuit(suit);
        bo.setMch(mch);
        bo.setParams(dto);
        bo.setStore(store);
        bo.setPlatform(deptService.selectDeptById(Constants.ROOT_DEPT));
        bo.setAgent(userService.selectSmUserByUserId(device.getAgentId()));
        return bo;
    }

    /**
     * 将参数转为提现BO
     *
     * @param dto
     */
    @Override
    public WithdrawBO toWithdrawBO(WithdrawDTO dto) {
        if (dto == null) {
            return null;
        }

        // 用户
        SmUserVo user = userService.selectSmUserByUserId(dto.getUserId());
        ServiceUtil.assertion(user == null, "用户不存在");

        // 渠道
        ChannelWithdrawVO channel = channelWithdrawService.selectChannelWithdrawByChannelId(dto.getChannelId());
        ServiceUtil.assertion(channel == null, "提现渠道不存在");
        ServiceUtil.assertion(channel.getEnabled() == null || !channel.getEnabled(), "提现渠道不可用");

        // 获取收款账户
        ServiceUtil.assertion(StringUtils.isBlank(channel.getAccountType()), "提现渠道没有对应的收款账户类型，请联系管理员处理");
        AccountQuery accountQuery = new AccountQuery();
        accountQuery.setUserId(user.getUserId());
        accountQuery.setAccountType(channel.getAccountType());
        AccountVO account = accountService.selectOne(accountQuery);
        ServiceUtil.assertion(account == null, "收款账户不存在");

        WithdrawBO bo = new WithdrawBO();
        bo.setDto(dto);
        bo.setUser(user);
        bo.setChannel(channel);
        bo.setAccount(account);
        return bo;
    }

    @Override
    public RechargePayBO toRechargePayBO(TransactionBillVO bill, Long channelId) {
        if (bill == null) {
            return null;
        }

        RechargePayBO bo = new RechargePayBO();
        bo.setOrder(bill);
        bo.setUser(userService.selectSmUserByUserId(bill.getUserId()));
        bo.setMch(userService.selectSmUserByUserId(bill.getMchId()));
        bo.setDevice(deviceService.selectSmDeviceByDeviceId(bill.getDeviceId()));
        bo.setChannel(channelService.selectSmChannelByChannelId(channelId));
        bo.setPlatform(deptService.selectDeptById(Constants.ROOT_DEPT));
        return bo;
    }

    @Override
    public RechargePayDepositBO toRechargePayDepositBO(PayDepositDTO dto) {
        if (dto == null) {
            return null;
        }
        TransactionBillVO order = transactionBillService.selectSmTransactionBillByBillNo(dto.getBillNo());

        RechargePayDepositBO bo = new RechargePayDepositBO();
        bo.setDto(dto);
        bo.setChannel(channelService.selectSmChannelByChannelId(dto.getChannelId()));
        bo.setOrder(order);
        if (order != null) {
            bo.setUser(userService.selectSmUserByUserId(order.getUserId()));
        }
        return bo;
    }

    @Override
    public EndUseBO toEndUseBO(EndUseDTO dto) {

        if (dto == null) {
            return null;
        }
        TransactionBillVO order = transactionBillService.selectSmTransactionBillByBillId(dto.getBillId());

        EndUseBO bo = new EndUseBO();
        bo.setOrder(order);
        bo.setTotalEle(dto.getTotalEle());
        if (order != null) {
            deviceService.pullDeviceInfo(order.getDeviceId());
            bo.setDevice(deviceService.selectSmDeviceByDeviceId(order.getDeviceId()));
        }
        return bo;
    }
}
